import { Controller, Get, Post, Body, Param, Delete, Query, Put, UseGuards, Request, Patch, HttpException, HttpStatus } from "@nestjs/common";
import { InjectModel } from "@nestjs/sequelize";
import { ShopService } from "./shop.service";
import { RoleGuard } from "src/users/guard/role.guard";
import { JwtAuthGuard } from "src/users/guard/auth.guard";
import { CreateProductDto } from "./dto/products/create-product.dto";
import { UpdateProductDto } from "./dto/products/update-product.dto";
import { UpdateCartDto } from "./dto/cart/update-cart.dto";
import { AddToCartDto } from "./dto/cart/add-to-cart.dto";
import { Product } from "./entities/product.entity";
import { Transaction } from "./entities/transaction.entity";
import { Payment } from "./entities/payment.entity";

@Controller("shop")
export class ShopController {
  constructor(
    private readonly shopService: ShopService,
    @InjectModel(Transaction) private readonly transactionModel: typeof Transaction,
    @InjectModel(Payment) private readonly paymentModel: typeof Payment,
  ) {}

  ////////////////////////////////////////products////////////////////////////////////////
  //create a new product (admin)
  @UseGuards(RoleGuard)
  @Post("product")
  async create(@Body() createProductDto: CreateProductDto) {
    const product = await this.shopService.create(createProductDto);
    return {
      message: "Product created successfully!",
      product,
    };
  }
  //list of all product
  @Get("product")
  async findAll(@Query() query: { search?: string; priceMin?: number; priceMax?: number }) {
    const { search, priceMin, priceMax } = query;
    return this.shopService.findAll(search, priceMin, priceMax);
  }
  //get a product detail
  @Get("product/:id")
  async findOne(@Param("id") id: string) {
    return this.shopService.findOne(id);
  }
  //edit a product info (admin)
  @UseGuards(RoleGuard)
  @Put("product/:id")
  async update(@Param("id") id: string, @Body() updateProductDto: UpdateProductDto) {
    const product = await this.shopService.update(id, updateProductDto);
    return {
      message: "product updated successful",
      product,
    };
  }
  //delete a product (admin)
  @UseGuards(RoleGuard)
  @Delete("product/:id")
  async remove(@Param("id") id: string){
    return this.shopService.remove(id);
  }
  ////////////////////////////////////////cart////////////////////////////////////////
  //create and a item to cart (user)
  @UseGuards(JwtAuthGuard) 
  @Post("cart")
  async createAndAddItemToCart(@Body() addToCartDto: AddToCartDto, @Request() req: any) {
    const userId = req.user.id;
    return this.shopService.createAndAddItemToCart({ ...addToCartDto, userId });
  }
  //get user cart items(user)
  @UseGuards(JwtAuthGuard)
  @Get("cart")
  async getUserOpenCart(@Request() req: any) {
    const userId = req.user.id;
    return this.shopService.getUserOpenCart(userId);
  }
  //edit quantity an item in cart (user)
  @UseGuards(JwtAuthGuard)
  @Patch("cart/:productId")
  async updateCart(@Param("productId") productId: number, @Body() updateCartDto: UpdateCartDto, @Request() req: any) {
    const userId = req.user.id;
    const updatedCart = await this.shopService.updateCart(userId, productId, updateCartDto.quantity);
    return {
      message: "Cart updated successfully",
      updatedCart,
    };
  }
  //delete an item from cart (user)
  @UseGuards(JwtAuthGuard)
  @Delete("cart/:productId")
  async removeFromCart(@Param("productId") productId: number, @Request() req: any) {
    const userId = req.user.id;
    return await this.shopService.removeFromCart(userId, productId);
  }
  //clear whole cart (user)
  @UseGuards(JwtAuthGuard)
  @Get("cart/clear-cart")
  async clearCart(@Request() req: any) {
    const userId = req.user.id;
    return await this.shopService.clearCart(userId);
  }
  //get checkout process (user)
  @UseGuards(JwtAuthGuard)
  @Get("cart/checkout")
  async processOrder(@Request() req: any) {
    const userId = req.user.id;
    try {
      const totalAmount = (await this.shopService.getUserOpenCart(userId)).totalPrice;
      const result = await this.shopService.processOrder(userId, totalAmount);
      return result;
    } catch (error) {
      throw new HttpException(error.message || "Order processing failed.", HttpStatus.INTERNAL_SERVER_ERROR);
    }
  }
  ////////////////////////////////////////wallet////////////////////////////////////////
  //getting wallet balance (user)
  @UseGuards(JwtAuthGuard)
  @Get("wallet")
  async getBalance(@Request() req) {
    const userId = req.user.id;
    return this.shopService.getBalance(userId);
  }
  //charging wallet (user)
  @UseGuards(JwtAuthGuard)
  @Post("wallet/charge")
  async addBalance(@Body("amount") amount: number, @Request() req) {
    const userId = req.user.id;
    const callbackUrl = `http://localhost:3000/payment/verify?userId=${userId}&amount=${amount}`;
    const paymentUrl = this.shopService.requestPayment(amount, "Wallet Charge", callbackUrl);
    return paymentUrl;
  }
  //get transaction (user)
  @UseGuards(JwtAuthGuard)
  @Get("wallet/transaction")
  async getTransactionById(@Request() req) {
    const userId = req.user.id;
    return this.shopService.getTransactionById(userId);
  }
  //get specific user transaction (admin)
  @UseGuards(RoleGuard)
  @Get("wallet/transaction/:id")
  async getTransactionByIdForAdmin(@Param("id") id: number) {
    return this.shopService.getTransactionByIdForAdmin(id);
  }
  ////////////////////////////////////////payment////////////////////////////////////////
  //payment request
  @UseGuards(JwtAuthGuard)
  @Get("payment/request")
  async requestPayment(@Request() req) {
    const userId = req.user.id;
    const invoice = await this.shopService.getInvoicePendingByUser(userId);
    const totalAmount = invoice.totalPaymentAmount;
    if (totalAmount < 1000) {
      return { message: "please enter amount above 1000" };
    }
    const callbackUrl = `http://localhost:3000/shop/payment/verify?userId=${userId}&amount=${totalAmount}`;
    const paymentUrl = await this.shopService.requestPayment(totalAmount, "Purchase products", callbackUrl);

    return { url: paymentUrl };
  }
  //payment verify
  @Get("payment/verify")
  async verifyPayment(@Query() query: { Authority: string; Status: string; userId: number; amount: number }): Promise<any> {
    const { Authority, Status, userId, amount } = query;

    if (Status !== "OK") {
      throw new Error("Payment failed");
    }

    if (!userId) {
      throw new Error("User ID is required.");
    }
    const wallet = this.shopService.getWalletInfo(userId);
    try {
      const refId = await this.shopService.verifyPayment(Authority, amount);
      await this.shopService.addBalance(userId, amount);
      const wallet = this.shopService.getWalletInfo(userId);
      await this.paymentModel.create({
        userId,
        walletId: (await wallet).walletId,
        paymentAmount: amount,
        status: "completed",
      });
      await this.transactionModel.create({
        walletId: (await wallet).walletId,
        amount: String(amount).startsWith("+") ? String(amount) : `+${amount}`,
      });
      return { message: "Payment successful", refId };
    } catch (error) {
      if(error instanceof HttpException){
        throw error
      }
      await this.paymentModel.create({
        userId,
        walletId: (await wallet).walletId,
        paymentAmount: amount,
        status: "failed",
      });
      throw new Error(`Error during payment verification: ${error.message}`);
    }
  }
  ////////////////////////////////////////invoice////////////////////////////////////////
  //get invoice (user)
  @UseGuards(JwtAuthGuard)
  @Get('invoice')
  async getInvoiceByUser(@Request() req) {
    const userId = req.user.id;
    return this.shopService.getInvoiceByUser(userId);
  }
  //get invoices list (admin)
  @UseGuards(RoleGuard)
  @Get('invoice/list')
  async getInvoices() {
    return this.shopService.getInvoices();
  }
  //get specific user invoices (admin)
  @UseGuards(RoleGuard)
  @Get('invoice/:id')
  async getUserInvoice(@Param('id') id:number) {
    return this.shopService.getUserInvoices(id);
  }

}
