import { Controller, Get, Query, Post, Body, UseGuards, Request, Put, Param, HttpException, HttpStatus, Delete } from "@nestjs/common";
import { UsersService } from "./users.service";
import { CreateUserDto } from "./dto/create-user.dto";
import { LoginUserDto } from "./dto/login-user.dto";
import { UpdateUserDto } from "./dto/update-user.dto";
import { JwtAuthGuard } from "./guard/auth.guard";
import { RoleGuard } from "./guard/role.guard";
import { User } from "./entities/user.entity";

@Controller("user")
export class UsersController {
  constructor(private readonly usersService: UsersService) {}
  // Register
  @Post("register")
  async register(@Body() createUserDto: CreateUserDto) {
    return this.usersService.register(createUserDto);
  }

  // Login
  @Post("login")
  async login(@Body() loginUserDto: LoginUserDto) {
    return this.usersService.login(loginUserDto);
  }

  // Get new access token
  @Post("new-token")
  async newAccessToken(@Body("token") token: string) {
    return this.usersService.newAccessToken(token);
  }

  // Logout user
  @UseGuards(JwtAuthGuard)
  @Get("logout")
  async logout(@Request() req) {
    const user = req.user;
    return this.usersService.logout(user);
  }

  // Retrieve user information
  @UseGuards(JwtAuthGuard)
  @Get()
  async getProfile(@Request() req): Promise<User> {
    const userId = req.user.id;
    return this.usersService.getProfile(userId);
  }

  // Edit user profile
  @UseGuards(JwtAuthGuard)
  @Put()
  async editProfile(@Request() req, @Body() updateUserDto: UpdateUserDto) {
    const userId = req.user.id;
    const { refreshToken, ...sanitizedDto } = updateUserDto;
    return this.usersService.editProfile(userId, sanitizedDto);
  }

  ///////////////////////////////////// Admin access endpoints /////////////////////////////////////////
  
  // Get users list (admin)
  @UseGuards(RoleGuard)
  @Get('users-list')
  async findAll(
    @Query("page") page: number = 1, // Default page is 1
    @Query("limit") limit: number = 10, // Default limit is 10
  ){
    return this.usersService.findAll(page, limit);
  }

  // Get specific user info (admin)
  @UseGuards(RoleGuard)
  @Get("userinfo/:id")
  async findSpecificUserInfoByUser(@Param("id") id) {
    return this.usersService.findSpecificUserInfoByUser(id);
  }

  // Delete specific user (admin)
  @UseGuards(RoleGuard)
  @Delete(":id?")
  async deleteUser(@Param("id") id) {
    if (!id) {
      throw new HttpException("User ID is required.", HttpStatus.BAD_REQUEST);
    }
    return this.usersService.deleteUser(id);
  }
}