import { HttpException, HttpStatus, Injectable, UnauthorizedException, BadRequestException, NotFoundException } from "@nestjs/common";
import { InjectModel } from "@nestjs/sequelize";
import { User } from "./entities/user.entity";
import * as bcrypt from "bcrypt";
import { JwtService } from "@nestjs/jwt";
import { ConfigService } from "@nestjs/config";
import { CreateUserDto } from "./dto/create-user.dto";
import { LoginUserDto } from "./dto/login-user.dto";
import { UpdateUserDto } from "./dto/update-user.dto";
import { v4 as uuidv4 } from "uuid";
import { RedisService } from "src/redis/redis.service";
import { Op } from "sequelize";

@Injectable()
export class UsersService {
  constructor(
    @InjectModel(User) private readonly userModel: typeof User,
    private readonly jwtService: JwtService,
    private readonly configService: ConfigService,
    private readonly redisService: RedisService,
  ) {
  }
  // Register method
  async register(createUserDto: CreateUserDto) {
    try {
      createUserDto.password = await bcrypt.hash(createUserDto.password, parseInt(process.env.BCRYPT_SALT_ROUNDS || "10", 10));

      const existingUser = await this.userModel.findOne({
        where: {
          [Op.or]: [{ email: createUserDto.email }, { phoneNumber: createUserDto.phoneNumber }, { username: createUserDto.username }],
        },
      });

      if (existingUser) {
        if (existingUser.email === createUserDto.email) {
          throw new BadRequestException("Email is already registered.");
        }
        if (existingUser.phoneNumber === createUserDto.phoneNumber) {
          throw new BadRequestException("Phone number is already registered.");
        }
        if (existingUser.username === createUserDto.username) {
          throw new BadRequestException("Username is already registered.");
        }
      }

      await this.userModel.create(createUserDto);
      const user = await this.userModel.findOne({
        where: { email: createUserDto.email },
      });
      return { message: "User registered successfully." };
    } catch (error) {
      if (error instanceof HttpException) {
        throw error;
      }
      throw new HttpException("An unexpected error occurred. Please try again later.", HttpStatus.INTERNAL_SERVER_ERROR);
    }
  }
  // Login method
  async login(loginUserDto: LoginUserDto) {
    try {
      const user = await this.userModel.findOne({
        where: { email: loginUserDto.email, username: loginUserDto.username },
      });

      if (!user) {
        throw new UnauthorizedException("Invalid email, username or password.");
      }

      const passwordMatch = await bcrypt.compare(loginUserDto.password, user.password);
      if (!passwordMatch) {
        throw new UnauthorizedException("Invalid email or password.");
      }

      const jti = uuidv4();
      const accessToken = this.jwtService.sign(
        { id: user.id, role: user.role, jti },
        {
          secret: this.configService.get<string>("JWT_SECRET"),
          expiresIn: this.configService.get<string | number>("JWT_EXPIRES") || "1h",
        },
      );

      const refreshToken = this.jwtService.sign(
        { id: user.id, jti },
        {
          secret: this.configService.get<string>("JWT_REFRESH_SECRET"),
          expiresIn: this.configService.get<string | number>("JWT_REFRESH_EXPIRES") || "7d",
        },
      );
      await this.redisService.addToWhitelist(user.id, jti, 3600);

      return { accessToken, refreshToken };
    } catch (error) {
      if (error instanceof HttpException) {
        throw error;
      }
      throw new HttpException("An unexpected error occurred. Please try again later.", HttpStatus.INTERNAL_SERVER_ERROR);
    }
  }
  // getting access token
  async newAccessToken(refreshToken: string) {
    if (!refreshToken) {
      throw new HttpException("Refresh token is required.", HttpStatus.BAD_REQUEST);
    }

    let decoded;
    try {
      decoded = this.jwtService.verify(refreshToken, { secret: this.configService.get<string>("JWT_REFRESH_SECRET") });
    } catch (error) {
      throw new HttpException("Invalid or expired token.", HttpStatus.UNAUTHORIZED);
    }

    const user = await this.getProfile(decoded.id);
    if (!user) {
      throw new HttpException("User not found.", HttpStatus.NOT_FOUND);
    }

    const jti = uuidv4();
    const accessToken = this.jwtService.sign(
      { id: user.id, role: user.role, jti },
      {
        secret: this.configService.get<string>("JWT_SECRET"),
        expiresIn: this.configService.get<string | number>("JWT_EXPIRES") || "1h",
      },
    );
    await this.redisService.addToWhitelist(user.id, jti, 3600);

    return { accessToken };
  }
  //logout (delete refresh token from database)
  async logout(user: any) {
    try {
      const jti = user.jti;
      const userId = user.id;

      await this.redisService.deleteFromWhitelist(userId, jti);

      return { message: "Logged out successfully." };
    } catch (error) {
      throw new UnauthorizedException("An error occurred during logout.");
    }
  }
  //get information user method
  async getProfile(userId: number) {
    try {
      const user = await this.userModel.findOne({
        where: { id: userId },
        attributes: { exclude: ["password", "refreshToken"] },
      });
      if (!user) {
        throw new HttpException("User not found.", HttpStatus.NOT_FOUND);
      }
      return user;
    } catch (error) {
      if (error instanceof HttpException) {
        throw error;
      }
      throw new HttpException("An unexpected error occurred. Please try again later.", HttpStatus.INTERNAL_SERVER_ERROR);
    }
  }
  //edit profile user method
  async editProfile(userId: number, updateUserDto: UpdateUserDto) {
    try {
      let user = await this.userModel.findOne({ where: { id: userId } });
      if (!user) {
        throw new NotFoundException("User not found.");
      }

      const { refreshToken, ...allowedUpdates } = updateUserDto;

      if (allowedUpdates.username) {
        const usernameExists = await this.userModel.findOne({
          where: { username: allowedUpdates.username },
        });
        if (usernameExists && usernameExists.id !== userId) {
          throw new BadRequestException("Username is already in use.");
        }
      }

      if (allowedUpdates.phoneNumber) {
        const phoneNumberExists = await this.userModel.findOne({
          where: { phoneNumber: allowedUpdates.phoneNumber },
        });
        if (phoneNumberExists && phoneNumberExists.id !== userId) {
          throw new BadRequestException("Phone number is already in use.");
        }
      }

      if (allowedUpdates.email) {
        const emailExists = await this.userModel.findOne({
          where: { email: allowedUpdates.email },
        });
        if (emailExists && emailExists.id !== userId) {
          throw new BadRequestException("Email is already in use.");
        }
      }

      if (allowedUpdates.password) {
        allowedUpdates.password = await bcrypt.hash(allowedUpdates.password, 10);
      }

      await user.update(allowedUpdates);

      user = await this.userModel.findOne({
        where: { id: userId },
        attributes: { exclude: ["password", "refreshToken"] },
      });

      return { message: "User account updated successfully.", user };
    } catch (error) {
      console.error(error);
      if (error instanceof HttpException) {
        throw error;
      }
      throw new HttpException("An unexpected error occurred. Please try again later.", HttpStatus.INTERNAL_SERVER_ERROR);
    }
  }
  //get users list
  async findAll(page: number = 1, limit: number = 10) {
    try {
      page = Math.max(page, 1);
      limit = Math.max(limit, 1);

      const offset = (page - 1) * limit;

      const users = await this.userModel.findAll({
        limit: limit,
        offset: offset,
        attributes: { exclude: ["refreshToken", "password", "email", "createdAt", "updatedAt", "gender", "phoneNumber"] },
      });

      return users;
    } catch (error) {
      if (error instanceof HttpException) {
        throw error;
      }
      throw new HttpException("An unexpected error occurred while fetching users.", HttpStatus.INTERNAL_SERVER_ERROR);
    }
  }
  //get users list
  async findSpecificUserInfoByUser(userId: number) {
    try {
      const user = await this.userModel.findByPk(userId, {
        attributes: { exclude: ["refreshToken", "password"] },
      });
      if (!user) {
        throw new HttpException("User not found.", HttpStatus.NOT_FOUND);
      }
      return user;
    } catch (error) {
      if (error instanceof HttpException) {
        throw error;
      }
      throw new HttpException("An unexpected error occurred while fetching user information.", HttpStatus.INTERNAL_SERVER_ERROR);
    }
  }
  //delete a specific user by admin
  async deleteUser(userId: number) {
    const user = await this.userModel.findOne({
      where: { id: userId },
    });
    if (!user) {
      throw new HttpException("User not found!.", HttpStatus.NOT_FOUND);
    }
    try {
      const deletedCount = await this.userModel.destroy({ where: { id: userId } });

      if (deletedCount === 0) {
        throw new HttpException("User not found.", HttpStatus.NOT_FOUND);
      }

      return { message: "User deleted successfully." };
    } catch (error) {
      if (error instanceof HttpException) {
        throw error;
      }
      throw new HttpException("An error occurred while deleting the user.", HttpStatus.INTERNAL_SERVER_ERROR);
    }
  }
}
